import 'package:mocobaas_pkg/HttpRequest.dart';
import 'package:mocobaas_pkg/Type/Auth/CheckEmail.dart';
import 'package:mocobaas_pkg/Type/Auth/GetOauth.dart';
import 'package:mocobaas_pkg/Type/Auth/Login.dart';
import 'package:mocobaas_pkg/Type/Auth/Register.dart';
import 'package:mocobaas_pkg/Type/Auth/ResetPassword.dart';
import 'package:mocobaas_pkg/mocobaas_pkg.dart';
import 'package:shared_preferences/shared_preferences.dart';

class Auth {
  late String appsUrl;
  late HttpRequestClass client;
  late String appsOrigin;
  late bool encryptMode = false;
  String prefixAccessToken = "MBaaS.auth.accessToken";
  String prefixRefreshToken = "MBaaS.auth.refreshToken";
  String prefixExpiryToken = "MBaaS.auth.expiryToken";

  Auth(String applicationUrl, MocoOpts? opt) {
    this.appsUrl = applicationUrl + "/services/auth";
    this.appsOrigin = applicationUrl;
    this.client = HttpRequestClass(appUrl: applicationUrl);
    if(opt != null) {
      var a = opt.options!.payloadEncryption!.enforcement!.contains("COMMON");
      print(a);
      this.encryptMode = a;
    }
  }

  /// Login Moco
  /// [data] {}
  Future<LoginResponse> login(Object data) async {
    try {
      var loginRes = await client.Request(appsUrl, "POST", data: data, encrypt: encryptMode);
      var response = LoginResponse.fromJson(loginRes.data);
      print(response);
      final prefs = await SharedPreferences.getInstance();

      /// save access and refresh token
      var att = response.data!.attributes!;
      await prefs.setString(prefixAccessToken, att.accessToken!);
      await prefs.setString(prefixRefreshToken, att.refreshToken!);
      await prefs.setString(prefixExpiryToken, att.expiry!);

      return response;
    } catch (e) {
      print(e);
      throw (e);
    }
  }

  Future<RegisterResponse> register(Object data) async {
    try {
      var registerRes =
          await client.Request(appsUrl + "/account", "POST", data: data, encrypt: encryptMode);
      var response = RegisterResponse.fromJson(registerRes.data);
      print(response);
      return response;
    } catch (e) {
      print(e);
      throw (e);
    }
  }

  Future<String> forgotPassword(String email) async {
    try {
      var forgotPass =
          await client.Request(appsUrl, "DELETE", params: {"email": email}, encrypt: encryptMode);
      print(forgotPass);
      return "OK";
    } catch (e) {
      print(e);
      throw (e);
    }
  }

  Future<CheckEmailResponse> checkEmail(String email) async {
    try {
      var check = await client.Request(appsUrl + "/account", "GET",
          params: {"email": email}, encrypt: encryptMode);
      var response = CheckEmailResponse.fromJson(check.data);
      return response;
    } catch (e) {
      print(e);
      throw (e);
    }
  }

  Future<String> resendVerification(String email) async {
    try {
      var resend = await client.Request(
          appsOrigin + "/services/emailVerification", "POST",
          data: {"email": email}, encrypt: encryptMode);
      print(resend.statusCode);
      return "OK";
    } catch (e) {
      print(e);
      throw (e);
    }
  }

  Future<ResetPasswordResponse> resetPassword(
      String token, String newPassword) async {
    try {
      var req = await client.Request(appsUrl, "PUT",
          data: {"token": token, "password": newPassword}, encrypt: encryptMode);
      print(req.statusCode);
      print(req);
      var response = ResetPasswordResponse.fromJson(req.data);
      return response;
    } catch (e) {
      print(e);
      throw (e);
    }
  }

  /// getOauthUrl : get oauth url for login social media
  /// [provider] string enum "facebook", "google", "twitter", "apple"
  Future<GetOuathResponse> getOauthUrl(String provider) async {
    try {
      var req = await client.Request(appsUrl, "GET",
          params: {"provider": provider, "callback": "http://localhost:8080"}, encrypt: encryptMode);
      print(req);
      var res = GetOuathResponse.fromJson(req.data);

      // save in prefs
      if (provider == "google" || provider == "facebook") {
        final prefs = await SharedPreferences.getInstance();
        var url = res.data!.attributes!.url;
        if (url != null) {
          prefs.setString(provider + "-Oauth", url);
        }
      }
      return res;
    } catch (e) {
      print(e);
      throw (e);
    }
  }

  /// redoOauth only for google and facebook
  Future<String?> redoOauth(String provider) async {
    try {
      if (provider == "google" || provider == "facebook") {
        final prefs = await SharedPreferences.getInstance();
        var getUrl = prefs.getString(provider + "-Oauth");
        return getUrl;
      } else {
        return null;
      }
    } catch (e) {
      print(e);
      throw (e);
    }
  }
}
