import 'dart:developer';

import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:mocobaas_pkg/Type/Auth/RefreshToken.dart';
import 'package:shared_preferences/shared_preferences.dart';

class HttpRequestClass {
  var dio = Dio();

  var defaultType = "application/vnd.api+json";
  var encryptionKeyLabel = "x-encryption-key";
  String prefixAccessToken = "MBaaS.auth.accessToken";
  String prefixRefreshToken = "MBaaS.auth.refreshToken";
  String prefixExpiryToken = "MBaaS.auth.expiryToken";
  var expiryThreshold = -50;
  late String appUrl;

  HttpRequestClass({String appUrl = ""}) {
    this.appUrl = appUrl;
    dio.interceptors.add(InterceptorsWrapper(
        onRequest: (options, handler) {
          //check expiry
          var auth = options.headers.containsKey("Authorization");
          if (auth) {
            try {
              var req = checkExpiry(options, handler);
            } on DioError catch (e) {
              print(e);
            }
          } else {
            return handler.next(options);
          }

          // Do something before request is sent
          // If you want to resolve the request with some custom data，
          // you can resolve a `Response` object eg: `handler.resolve(response)`.
          // If you want to reject the request with a error message,
          // you can reject a `DioError` object eg: `handler.reject(dioError)`
        },
        onResponse: (response, handler) {
          // Do something with response data
          print(response);
          return handler.next(response); // continue
          // If you want to reject the request with a error message,
          // you can reject a `DioError` object eg: `handler.reject(dioError)`
        },
        onError: (DioError e, handler) {
          // Do something with response error
          var code = e.response!.statusCode;
          print(code);
          if(code == 401) {
            print("Error Access Token");
            var title = e.response!.data["errors"]![0];
            if(title["code"] == "IVACC"){
              handleError(e, handler);
            } else {
              return handler.next(e); //continue
            }
          } else {
            return handler.next(e); //continue
            // If you want to resolve the request with some custom data，
            // you can resolve a `Response` object eg: `handler.resolve(response)`.
          }
        }
    ));
  }

  Future<void> checkExpiry(RequestOptions opts, RequestInterceptorHandler handler) async {
    //get current token expiry
    print("CHECK EXP");
    var dio2 = Dio();
    final prefs = await SharedPreferences.getInstance();
    var expCurrent = DateTime.parse(prefs.getString(prefixExpiryToken)!);
    var timenow = DateTime.now();

    print(expCurrent);
    print(timenow);

    var gep = timenow
        .difference(expCurrent)
        .inSeconds;
    print(gep);
    if (gep > expiryThreshold) {
      print("Must Reload");
        var rt = prefs.getString(prefixRefreshToken);
        print(rt);
      try {
        print("HERE");
        var response = await dio2.request(
          appUrl + "/services/auth",
          data: {
            "refreshToken": rt
          },
          options: Options(method: "PATCH", headers: {
            "accept": defaultType,
            Headers.contentTypeHeader: defaultType
          }),
        );
        print(response.requestOptions);
        print(response);
        var res = RefreshTokenResponse.fromJson(response.data);

        //set new token & expiry
        prefs.setString(prefixRefreshToken, res.data!.attributes!.refreshToken!);
        prefs.setString(prefixAccessToken, res.data!.attributes!.accessToken!);
        prefs.setString(prefixExpiryToken, res.data!.attributes!.expiry!);

        var atNew = res.data!.attributes!.accessToken!;
        opts.headers.update("Authorization", (value) => "Bearer $atNew");
        handler.next(opts);
      } on DioError catch(e) {
        print(e.message);
        handler.reject(e);
      }
    } else {
      handler.next(opts);
    }
  }

  Future<void> handleError(DioError e, ErrorInterceptorHandler handler) async{
    try {
      var dio2 = Dio();
      final prefs = await SharedPreferences.getInstance();
      var rt = prefs.getString(prefixRefreshToken);
      var response = await dio2.request(
        appUrl + "/services/auth",
        data: {
          "refreshToken": rt
        },
        options: Options(method: "PATCH", headers: {
          "accept": defaultType,
          Headers.contentTypeHeader: defaultType
        }),
      );
      print(response.requestOptions);
      print(response);
      var res = RefreshTokenResponse.fromJson(response.data);

      //set new token & expiry
      prefs.setString(prefixRefreshToken, res.data!.attributes!.refreshToken!);
      prefs.setString(prefixAccessToken, res.data!.attributes!.accessToken!);
      prefs.setString(prefixExpiryToken, res.data!.attributes!.expiry!);

      var atNew = res.data!.attributes!.accessToken!;

      //retry new request

      //set bearer
      e.requestOptions.headers["Authorization"] =
          "Bearer " + atNew;
      //create request with new access token
      final opts = new Options(
          method: e.requestOptions.method,
          headers: e.requestOptions.headers);
      final cloneReq = await dio.request(e.requestOptions.path,
          options: opts,
          data: e.requestOptions.data,
          queryParameters: e.requestOptions.queryParameters);
      return handler.resolve(cloneReq);
    } on DioError catch(e) {
      print(e.message);
      handler.reject(e);
    }
  }


  Future<Response<dynamic>> Request(String url, String method,
      {Object? data, Map<String,
          dynamic>? params, bool auth: false, bool encrypt = false}) async {
    try {
      print("Request $method $url");

      // set auth
      if (auth) {
        final prefs = await SharedPreferences.getInstance();
        var at = prefs.getString("MBaaS.auth.accessToken");
        if (at == null) throw ("Authentication required!");
        dio.options = BaseOptions(headers: {
          "Authorization": "Bearer $at",
          "accept": defaultType,
          Headers.contentTypeHeader: defaultType
        });
      } else {
        dio.options = BaseOptions(
          headers: {
            "accept": defaultType,
            Headers.contentTypeHeader: defaultType
          },
          // followRedirects: false,
          // // will not throw errors
          // validateStatus: (status) => true
        );
      }

      print(dio.options.headers);

      if (encrypt) {

      }

      var response = await dio.request(
        url,
        data: data,
        queryParameters: params,
        options: Options(method: method),
      );

      return response;
    } on DioError catch (e) {
      print("error get");
      if (e.response != null) {
        print("res != null");
        print(e.response!.data);
        print(e.response!.headers);
        print(e.response!.requestOptions);
        throw (e.response!.data);
      } else {
        // Something happened in setting up or sending the request that triggered an Error
        print("error in first request");
        print(e.requestOptions);
        print(e.message);
        throw (e.message);
      }
    }
  }
}
