
import 'dart:convert';

import 'package:mocobaas_pkg/Type/Storage/Download.dart';
import 'package:mocobaas_pkg/Type/Storage/Remove.dart';
import 'package:mocobaas_pkg/Type/Storage/Upload.dart';

import 'HttpRequest.dart';
class Storage {
  late String appsUrl;
  late HttpRequestClass client;
  String prefixAccessToken = "MBaaS.auth.accessToken";
  String prefixRefreshToken = "MBaaS.auth.refreshToken";

  Storage(String applicationUrl) {
    this.appsUrl = applicationUrl + "/services";
    this.client = HttpRequestClass();
  }

  Future<UploadUrlResponse> uploadUrl(String bucket, String expire, String filename) async {
    try {
      var req = await client.Request(appsUrl+"/uploadUrl","GET", auth: true, params: {
        "bucket" : bucket,
        "expire" : expire,
        "filename" : filename
      });
      var res = UploadUrlResponse.fromJson(req.data);
      return res;
    } catch (e) {
      print(e);
      throw(e);
    }
  }

  Future<DownloadUrlResponse> downloadUrl(String bucket, String expire, String fileId, {bool signed = false}) async {
    try {
      var req = await client.Request(appsUrl+"/downloadUrl","GET", auth: true, params: {
        "bucket" : bucket,
        "expire" : expire,
        "signed" : signed,
        "fileId" : fileId
      });
      var res = DownloadUrlResponse.fromJson(req.data);
      return res;
    } catch (e) {
      print(e);
      throw(e);
    }
  }

  Future<RemoveFileResponse> removeFile(String bucket, String filePath) async {
    try {
      var req = await client.Request(appsUrl+"/removeFile","GET", auth: true,params: {
        "bucket" : bucket,
        "filePath" : filePath
      });
      var res = RemoveFileResponse.fromJson(req.data);
      return res;
    } catch (e) {
      print(e);
      throw(e);
    }
  }

}