
import 'package:mocobaas_pkg/Payload.dart';
import 'package:mocobaas_pkg/Type/User/GetUser.dart';
import 'package:mocobaas_pkg/Type/User/UpdateUser.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'HttpRequest.dart';

class User {
  late String appsUrl;
  late HttpRequestClass client;
  String prefixAccessToken = "MBaaS.auth.accessToken";
  String prefixRefreshToken = "MBaaS.auth.refreshToken";

  User(String applicationUrl) {
    this.appsUrl = applicationUrl + "/services/me";
    this.client = HttpRequestClass(appUrl: applicationUrl);
  }

  Future<String?> logout() async {
    try {
      var logout = await client.Request(appsUrl,"DELETE", auth: true);
      print(logout);
      final prefs = await SharedPreferences.getInstance();
      prefs.remove(prefixRefreshToken);
      prefs.remove(prefixAccessToken);
      return "OK";
    } catch (e) {
      print(e);
      throw(e);
    }
  }

  Future<GetUserResponse> get() async {
    try {
      var getUser = await client.Request(appsUrl,"GET", auth: true);
      var response = GetUserResponse.fromJson(getUser.data);
      print(response);
      return response;
    } catch (e) {
      print(e);
      throw(e);
    }
  }

  Future<UpdateUserResponse> update(Object data) async {
    try {
      var req = await client.Request(appsUrl,"PATCH", auth: true, data: data);
      var response = UpdateUserResponse.fromJson(req.data);
      print(response);
      return response;
    } catch (e) {
      print(e);
      throw(e);
    }
  }

  Future<bool> isLoggedIn() async{
    var a = Payload(appsUrl);
    a.generate();

    final prefs = await SharedPreferences.getInstance();
    var check = prefs.getString(prefixAccessToken);
    return check != null;
  }



}