library mocobaas_pkg;

import 'package:mocobaas_pkg/Graphql.dart';
import 'package:mocobaas_pkg/Storage.dart';
import 'package:shared_preferences/shared_preferences.dart';

import 'Auth.dart';

import 'Invoke.dart';
import 'User.dart';

/// A Calculator.
class Moco {

  late String appsUrl;
  late Auth auth;
  late User user;
  late Invoke _invoke;
  late Graphql gql;
  late Storage storage;

  late PayloadQuery payloadQuery;
  String prefixAccessToken = "MBaaS.auth.accessToken";
  String prefixRefreshToken = "MBaaS.auth.refreshToken";
  String prefixExpiry = "MBaaS.auth.expiry";

  /// Constructor
  Moco(String applicationUrl, {MocoOpts? options }) {
    this.appsUrl = applicationUrl;
    this.auth = Auth(applicationUrl, options);
    this.user = User(applicationUrl);
    this._invoke = Invoke(applicationUrl);
    this.gql = Graphql(applicationUrl);
    this.storage = Storage(applicationUrl);
  }



  /// Returns [value] plus 1.
  // ResponseAddOne addOne(int value) {
  //  var hasil = value + 1;
  // return ResponseAddOne("OK", hasil);
  // }

  Future<String> saveTokens(String? accessToken, String? refreshToken, String? expiry) async {
    try {
      final prefs = await SharedPreferences.getInstance();
      if (accessToken != null) {
        prefs.setString(prefixAccessToken, accessToken);
      }
      if (refreshToken != null) {
        prefs.setString(prefixRefreshToken, refreshToken);
      }
      if (expiry != null) {
        prefs.setString(prefixExpiry, expiry);
      }
      return "OK";
    } catch(e) {
      print(e);
      throw(e);
    }
  }

  Future<dynamic> invokeApi(String apiName, Object? invokeData) async {
    try {
      var invokeMethod = await _invoke.invokeMethod(apiName, invokeData);
      return invokeMethod;
    } catch(e) {
      print(e);
      throw(e);
    }
  }

}

// class ResponseAddOne {
//   ResponseAddOne(this.message, this.count);
//   final String message;
//   final int count;
//   @override
//   String toString() {
//     return 'Student: {message: $message, age: $count}';
//   }
// }

class MocoOpts {
  Options? options;

  MocoOpts({this.options});

  MocoOpts.fromJson(Map<String, dynamic> json) {
    options =
    json['options'] != null ? new Options.fromJson(json['options']) : null;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    if (this.options != null) {
      data['options'] = this.options!.toJson();
    }
    return data;
  }
}

class Options {
  PayloadEncryption? payloadEncryption;

  Options({this.payloadEncryption});

  Options.fromJson(Map<String, dynamic> json) {
    payloadEncryption = json[' payloadEncryption'] != null
        ? new PayloadEncryption.fromJson(json[' payloadEncryption'])
        : null;
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    if (this.payloadEncryption != null) {
      data[' payloadEncryption'] = this.payloadEncryption!.toJson();
    }
    return data;
  }
}

class PayloadEncryption {
  List<String>? enforcement;

  PayloadEncryption({this.enforcement});

  PayloadEncryption.fromJson(Map<String, dynamic> json) {
    enforcement = json['enforcement'].cast<String>();
  }

  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = new Map<String, dynamic>();
    data['enforcement'] = this.enforcement;
    return data;
  }
}
